@@ -17,9 +17,10 @@ from django.contrib.auth import authenticate
 from .store import DjangoOpenIDStore
 from .backends import TwitterBackend, OrkutBackend, FacebookBackend, \
                       OpenIDBackend, GoogleBackend, YahooBackend, \
-                      GoogleOAuthBackend
+                      GoogleOAuthBackend, LiveJournalBackend
 from .conf import AX_ATTRS, SREG_ATTR, OPENID_ID_FIELD, SESSION_NAME, \
                   OPENID_GOOGLE_URL, OPENID_YAHOO_URL, TWITTER_SERVER, \
+                  OPENID_LIVEJOURNAL_URL, OPENID_LIVEJOURNAL_USER_FIELD, \
                   TWITTER_REQUEST_TOKEN_URL, TWITTER_ACCESS_TOKEN_URL, \
                   TWITTER_AUTHORIZATION_URL, TWITTER_CHECK_AUTH, \
                   FACEBOOK_CHECK_AUTH, FACEBOOK_AUTHORIZATION_URL, \
@@ -168,6 +169,23 @@ class YahooAuth(OpenIdAuth):
         return OPENID_YAHOO_URL
 
 
+class LiveJournalAuth(OpenIdAuth):
+    """LiveJournal OpenID authentication"""
+    AUTH_BACKEND = LiveJournalBackend
+
+    def uses_redirect(self):
+        """LiveJournal uses redirect"""
+        return True
+
+    def openid_url(self):
+        """Returns LiveJournal authentication URL"""
+        if self.request.method != 'POST' or \
+           not self.request.POST.get(OPENID_LIVEJOURNAL_USER_FIELD):
+            raise ValueError, 'Missing LiveJournal user identifier'
+        return OPENID_LIVEJOURNAL_URL % \
+                    self.request.POST[OPENID_LIVEJOURNAL_USER_FIELD]
+
+
 class BaseOAuth(BaseAuth):
     """OAuth base class"""
     def __init__(self, request, redirect):
@@ -395,16 +413,13 @@ class TwitterAuth(ConsumerBasedOAuth):
 
 class FacebookAuth(BaseOAuth):
     """Facebook OAuth mechanism"""
-    def __init__(self, request, redirect):
-        super(FacebookAuth, self).__init__(request, redirect)
-        if settings.DEBUG and self.redirect_uri:
-            # Facebook doesn't accept custom ports
-            self.redirect_uri = self.redirect_uri.replace(':8000', '')
 
     def auth_url(self):
         """Returns redirect url"""
         args = {'client_id': settings.FACEBOOK_APP_ID,
                 'redirect_uri': self.redirect_uri}
+        if hasattr(settings, 'FACEBOOK_EXTENDED_PERMISSIONS'):
+            args['scope'] = ','.join(settings.FACEBOOK_EXTENDED_PERMISSIONS)
         return FACEBOOK_AUTHORIZATION_URL + '?' + urllib.urlencode(args)
 
     def auth_complete(self, *args, **kwargs):
@@ -446,6 +461,7 @@ BACKENDS = {
     'google': GoogleAuth,
     'google-oauth': GoogleOAuth,
     'yahoo': YahooAuth,
+    'livejournal': LiveJournalAuth,
     'orkut': OrkutAuth,
     'openid': OpenIdAuth,
 }
